"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) if (e.indexOf(p[i]) < 0)
            t[p[i]] = s[p[i]];
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var redux_data_service_1 = require("redux-data-service");
var recompose_1 = require("recompose");
var lodash_1 = require("lodash");
var Query_1 = require("../Query");
var WithModelQuery_1 = require("../WithModelQuery");
var Helpers_1 = require("../Helpers");
var WithDelayedHandlers_1 = require("../WithDelayedHandlers");
var InfiniteScrollPreviousPage_1 = require("./InfiniteScrollPreviousPage");
var DefaultContentPlaceHolder = function (_a) {
    var height = _a.height;
    var style = {
        height: height,
    };
    return (React.createElement("div", { style: style }));
};
exports.InfiniteScroll = recompose_1.compose(recompose_1.setDisplayName("InfiniteScroll"), recompose_1.defaultProps({
    contentPlaceHolderComponent: DefaultContentPlaceHolder,
}), recompose_1.withStateHandlers(function (_a) {
    var query = _a.query, modelName = _a.modelName;
    return ({
        currentItems: null,
        currentPage: 1,
        currentPageStartMarkerRef: React.createRef(),
        currentPageEndMarkerRef: React.createRef(),
        hasNextPage: false,
        hasPreviousPage: false,
        lastScrollTop: 0,
        nextPage: null,
        nextPageEndMarkerRef: React.createRef(),
        nextPageQuery: null,
        nextPageStartMarkerRef: React.createRef(),
        nextPlaceHolderHeight: 0,
        pageHeightMap: {},
        previousPage: null,
        previousPageEndMarkerRef: React.createRef(),
        previousPageQuery: null,
        previousPageStartMarkerRef: React.createRef(),
        previousPlaceHolderHeight: 0,
        query: query instanceof redux_data_service_1.QueryBuilder
            ? query
            : new redux_data_service_1.QueryBuilder(modelName, query),
        totalPages: 0,
    });
}, {
    updateState: function () { return function (newState) { return newState; }; },
    updateStateFromQuery: function () { return function (queryManager) { return ({
        currentItems: queryManager.items,
        currentPage: queryManager.response.currentPage,
        nextPage: queryManager.response.nextPage,
        nextPageQuery: queryManager.getNextPage(),
        hasNextPage: queryManager.hasNextPage(),
        hasPreviousPage: queryManager.hasPreviousPage(),
        previousPage: queryManager.response.previousPage,
        previousPageQuery: queryManager.getPreviousPage(),
        totalPages: queryManager.response.totalPages,
    }); }; },
}), WithModelQuery_1.withModelQuery(), recompose_1.withPropsOnChange(["currentPage", "pageHeightMap", "totalPages"], function (_a) {
    var currentPage = _a.currentPage, pageHeightMap = _a.pageHeightMap, totalPages = _a.totalPages;
    var previousPlaceHolderHeight = 0;
    var nextPlaceHolderHeight = 0;
    var estimatedPageHeight = Helpers_1.average(Object.values(pageHeightMap));
    for (var x = currentPage - 2; x > 0; x--) {
        previousPlaceHolderHeight += x in pageHeightMap
            ? pageHeightMap[x]
            : estimatedPageHeight;
    }
    for (var x = currentPage + 2; x <= totalPages; x++) {
        nextPlaceHolderHeight += x in pageHeightMap
            ? pageHeightMap[x]
            : estimatedPageHeight;
    }
    return {
        estimatedPageHeight: estimatedPageHeight,
        nextPlaceHolderHeight: nextPlaceHolderHeight,
        previousPlaceHolderHeight: previousPlaceHolderHeight,
    };
}), WithDelayedHandlers_1.withDelayedHandlers({
    updatePageHeightMap: function (_a) {
        var currentPage = _a.currentPage, currentPageEndMarkerRef = _a.currentPageEndMarkerRef, currentPageStartMarkerRef = _a.currentPageStartMarkerRef, hasNextPage = _a.hasNextPage, hasPreviousPage = _a.hasPreviousPage, nextPage = _a.nextPage, nextPageEndMarkerRef = _a.nextPageEndMarkerRef, nextPageStartMarkerRef = _a.nextPageStartMarkerRef, pageHeightMap = _a.pageHeightMap, previousPage = _a.previousPage, previousPageEndMarkerRef = _a.previousPageEndMarkerRef, previousPageStartMarkerRef = _a.previousPageStartMarkerRef, updateState = _a.updateState;
        return function () { return function () {
            var updatedPageHeightMap = {};
            if (currentPageStartMarkerRef.current) {
                var currentPageHeight = Helpers_1.calculateGroupHeight(currentPageStartMarkerRef.current.nextSibling, currentPageEndMarkerRef.current.previousSibling);
                if (currentPageHeight !== pageHeightMap[currentPage]) {
                    updatedPageHeightMap[currentPage] = currentPageHeight;
                }
            }
            if (hasPreviousPage && previousPageStartMarkerRef.current) {
                var previousPageHeight = Helpers_1.calculateGroupHeight(previousPageStartMarkerRef.current.nextSibling, previousPageEndMarkerRef.current.previousSibling);
                if (previousPageHeight !== pageHeightMap[previousPage]) {
                    updatedPageHeightMap[previousPage] = previousPageHeight;
                }
            }
            if (hasNextPage && nextPageStartMarkerRef.current) {
                var nextPageHeight = Helpers_1.calculateGroupHeight(nextPageStartMarkerRef.current.nextSibling, nextPageEndMarkerRef.current.previousSibling);
                if (nextPageHeight !== pageHeightMap[nextPage]) {
                    updatedPageHeightMap[nextPage] = nextPageHeight;
                }
            }
            if (!lodash_1.isEmpty(updatedPageHeightMap)) {
                pageHeightMap = __assign({}, pageHeightMap, updatedPageHeightMap);
                updateState({
                    pageHeightMap: pageHeightMap
                });
            }
        }; };
    },
    handleScroll: function (_a) {
        var currentPage = _a.currentPage, disableVirtualScrolling = _a.disableVirtualScrolling, estimatedPageHeight = _a.estimatedPageHeight, hasNextPage = _a.hasNextPage, lastScrollTop = _a.lastScrollTop, nextPageQuery = _a.nextPageQuery, pageHeightMap = _a.pageHeightMap, query = _a.query, totalPages = _a.totalPages, updateState = _a.updateState;
        return function (_a) {
            var _b = _a.target, clientHeight = _b.clientHeight, scrollHeight = _b.scrollHeight, scrollTop = _b.scrollTop;
            return function () {
                var updatedState = {
                    lastScrollTop: scrollTop
                };
                var scrollingDown = scrollTop > lastScrollTop;
                var currentScrollBottom = scrollHeight - scrollTop - clientHeight;
                if (!disableVirtualScrolling) {
                    var nextPageToLoad = 1;
                    var nextPageScrollTop = pageHeightMap[1];
                    while (nextPageScrollTop < (scrollTop + (clientHeight / 2)) && nextPageToLoad < totalPages) {
                        nextPageToLoad++;
                        nextPageScrollTop += nextPageToLoad in pageHeightMap
                            ? pageHeightMap[nextPageToLoad]
                            : estimatedPageHeight;
                    }
                    if (currentPage !== nextPageToLoad) {
                        updatedState.query = query.query.page(nextPageToLoad);
                    }
                }
                else if (hasNextPage && scrollingDown && currentScrollBottom < (clientHeight / 2)) {
                    updatedState.query = nextPageQuery;
                }
                updateState(updatedState);
            };
        };
    },
}), recompose_1.pure, recompose_1.lifecycle({
    componentDidMount: function () {
        if (this.props.query.response) {
            this.props.updateStateFromQuery(this.props.query);
        }
    },
    componentDidUpdate: function (prevProps) {
        if (prevProps.query !== this.props.query && this.props.query.response) {
            this.props.updateStateFromQuery(this.props.query);
        }
        else if (this.props.currentItems && this.props.pageHeightMap === prevProps.pageHeightMap) {
            this.props.updatePageHeightMap();
        }
    },
}), Helpers_1.omitProps([
    "currentPage",
    "items",
    "lastScrollTop",
    "loadingComponent",
    "loadingComponentProps",
    "totalPages",
    "updatePageHeightMap",
    "updateState",
    "updateStateFromQuery",
]))(function (_a) {
    var ContainerComponent = _a.containerComponent, ContentPlaceHolder = _a.contentPlaceHolderComponent, contentPlaceHolderComponentProps = _a.contentPlaceHolderComponentProps, currentItems = _a.currentItems, currentPageEndMarkerRef = _a.currentPageEndMarkerRef, currentPageStartMarkerRef = _a.currentPageStartMarkerRef, disableVirtualScrolling = _a.disableVirtualScrolling, estimatedPageHeight = _a.estimatedPageHeight, handleScroll = _a.handleScroll, hasNextPage = _a.hasNextPage, hasPreviousPage = _a.hasPreviousPage, ModelComponent = _a.modelComponent, modelComponentProps = _a.modelComponentProps, modelName = _a.modelName, nextPage = _a.nextPage, nextPageEndMarkerRef = _a.nextPageEndMarkerRef, nextPageQuery = _a.nextPageQuery, nextPageStartMarkerRef = _a.nextPageStartMarkerRef, nextPlaceHolderHeight = _a.nextPlaceHolderHeight, pageHeightMap = _a.pageHeightMap, previousPage = _a.previousPage, previousPageEndMarkerRef = _a.previousPageEndMarkerRef, previousPageQuery = _a.previousPageQuery, previousPlaceHolderHeight = _a.previousPlaceHolderHeight, previousPageStartMarkerRef = _a.previousPageStartMarkerRef, containerProps = __rest(_a, ["containerComponent", "contentPlaceHolderComponent", "contentPlaceHolderComponentProps", "currentItems", "currentPageEndMarkerRef", "currentPageStartMarkerRef", "disableVirtualScrolling", "estimatedPageHeight", "handleScroll", "hasNextPage", "hasPreviousPage", "modelComponent", "modelComponentProps", "modelName", "nextPage", "nextPageEndMarkerRef", "nextPageQuery", "nextPageStartMarkerRef", "nextPlaceHolderHeight", "pageHeightMap", "previousPage", "previousPageEndMarkerRef", "previousPageQuery", "previousPlaceHolderHeight", "previousPageStartMarkerRef"]);
    return (React.createElement(ContainerComponent, __assign({}, containerProps, { onScroll: handleScroll }),
        hasPreviousPage && disableVirtualScrolling && (React.createElement(InfiniteScrollPreviousPage_1.InfiniteScrollPreviousPage, { modelComponent: ModelComponent, modelComponentProps: modelComponentProps, query: previousPageQuery })),
        hasPreviousPage && !disableVirtualScrolling && (React.createElement(ContentPlaceHolder, __assign({ height: previousPlaceHolderHeight }, contentPlaceHolderComponentProps))),
        hasPreviousPage && !disableVirtualScrolling && (React.createElement("script", { ref: previousPageStartMarkerRef })),
        hasPreviousPage && !disableVirtualScrolling && (React.createElement(Query_1.Query, { modelName: modelName, query: previousPageQuery, loadingComponent: ContentPlaceHolder, loadingComponentProps: { height: pageHeightMap[previousPage] || estimatedPageHeight } }, function (_a) {
            var query = _a.query;
            return (query.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); }));
        })),
        hasPreviousPage && !disableVirtualScrolling && (React.createElement("script", { ref: previousPageEndMarkerRef })),
        React.createElement("script", { ref: currentPageStartMarkerRef }),
        (currentItems == null
            ? React.createElement(ContentPlaceHolder, __assign({ height: "100%" }, contentPlaceHolderComponentProps))
            : currentItems.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); })),
        React.createElement("script", { ref: currentPageEndMarkerRef }),
        hasNextPage && (React.createElement("script", { ref: nextPageStartMarkerRef })),
        hasNextPage && (React.createElement(Query_1.Query, { modelName: modelName, query: nextPageQuery, loadingComponent: ContentPlaceHolder, loadingComponentProps: { height: pageHeightMap[nextPage] || estimatedPageHeight } }, function (_a) {
            var query = _a.query;
            return (query.items.map(function (model) { return (React.createElement(ModelComponent, __assign({ key: model.id, model: model }, modelComponentProps))); }));
        })),
        hasNextPage && (React.createElement("script", { ref: nextPageEndMarkerRef })),
        hasNextPage && !disableVirtualScrolling && (React.createElement(ContentPlaceHolder, __assign({ height: nextPlaceHolderHeight }, contentPlaceHolderComponentProps)))));
});
